@@ -3,11 +3,12 @@
 /**
  * Set of functions used with the bookmark feature
  *
- * @version $Id: bookmark.lib.php 11453 2008-08-01 20:35:32Z lem9 $
+ * @version $Id$
+ * @package phpMyAdmin
  */
 
 /**
- * 
+ *
  */
 require_once './libraries/relation.lib.php';
 
@@ -25,11 +26,11 @@ require_once './libraries/relation.lib.php';
 function PMA_Bookmark_getParams()
 {
     static $cfgBookmark = null;
-    
+
     if (null !== $cfgBookmark) {
         return $cfgBookmark;
     }
-    
+
     $cfgRelation = PMA_getRelationsParam();
 
     if ($cfgRelation['bookmarkwork']) {
@@ -41,7 +42,7 @@ function PMA_Bookmark_getParams()
     } else {
         $cfgBookmark = false;
     }
-    
+
     return $cfgBookmark;
 } // end of the 'PMA_Bookmark_getParams()' function
 
@@ -58,14 +59,14 @@ function PMA_Bookmark_getParams()
  *
  * @param   string    the current database name
  *
- * @return  array     the bookmarks list (key as index, label as value) 
+ * @return  array     the bookmarks list (key as index, label as value)
  *
  * @access  public
  */
 function PMA_Bookmark_getList($db)
 {
     global $controllink;
-    
+
     $cfgBookmark = PMA_Bookmark_getParams();
 
     if (empty($cfgBookmark)) {
@@ -94,26 +95,35 @@ function PMA_Bookmark_getList($db)
  * @param   mixed     the id of the bookmark to get
  * @param   string    which field to look up the $id
  * @param   boolean  TRUE: get all bookmarks regardless of the owning user
+ * @param   boolean   whether to ignore bookmarks with no user
  *
  * @return  string    the sql query
  *
  * @access  public
  */
-function PMA_Bookmark_get($db, $id, $id_field = 'id', $action_bookmark_all = FALSE)
+function PMA_Bookmark_get($db, $id, $id_field = 'id', $action_bookmark_all = FALSE, $exact_user_match = FALSE)
 {
     global $controllink;
-    
+
     $cfgBookmark = PMA_Bookmark_getParams();
 
     if (empty($cfgBookmark)) {
         return '';
     }
-    
+
     $query = 'SELECT query FROM ' . PMA_backquote($cfgBookmark['db']) . '.' . PMA_backquote($cfgBookmark['table'])
-        . ' WHERE dbase = \'' . PMA_sqlAddslashes($db) . '\''
-        . ($action_bookmark_all? '' : ' AND (user = \'' . PMA_sqlAddslashes($cfgBookmark['user']) . '\''
-        . '      OR user = \'\')')
-        . ' AND ' . PMA_backquote($id_field) . ' = ' . $id;
+        . ' WHERE dbase = \'' . PMA_sqlAddslashes($db) . '\'';
+
+    if (!$action_bookmark_all) {
+        $query .= ' AND (user = \'' . PMA_sqlAddslashes($cfgBookmark['user']) . '\'';
+        if (!$exact_user_match) {
+            $query .= ' OR user = \'\'';
+        }
+        $query .= ')';
+    }
+
+    $query .= ' AND ' . PMA_backquote($id_field) . ' = ' . $id;
+
     return PMA_DBI_fetch_value($query, 0, 0, $controllink);
 } // end of the 'PMA_Bookmark_get()' function
 
@@ -137,13 +147,13 @@ function PMA_Bookmark_get($db, $id, $id_field = 'id', $action_bookmark_all = FAL
 function PMA_Bookmark_save($fields, $all_users = false)
 {
     global $controllink;
-    
+
     $cfgBookmark = PMA_Bookmark_getParams();
 
     if (empty($cfgBookmark)) {
         return false;
     }
-    
+
     $query = 'INSERT INTO ' . PMA_backquote($cfgBookmark['db']) . '.' . PMA_backquote($cfgBookmark['table'])
            . ' (id, dbase, user, query, label) VALUES (NULL, \'' . PMA_sqlAddslashes($fields['dbase']) . '\', \'' . ($all_users ? '' : PMA_sqlAddslashes($fields['user'])) . '\', \'' . PMA_sqlAddslashes(urldecode($fields['query'])) . '\', \'' . PMA_sqlAddslashes($fields['label']) . '\')';
     return PMA_DBI_query($query, $controllink);
@@ -167,13 +177,13 @@ function PMA_Bookmark_save($fields, $all_users = false)
 function PMA_Bookmark_delete($db, $id)
 {
     global $controllink;
-    
+
     $cfgBookmark = PMA_Bookmark_getParams();
 
     if (empty($cfgBookmark)) {
         return false;
     }
-    
+
     $query  = 'DELETE FROM ' . PMA_backquote($cfgBookmark['db']) . '.' . PMA_backquote($cfgBookmark['table'])
             . ' WHERE (user = \'' . PMA_sqlAddslashes($cfgBookmark['user']) . '\''
             . '        OR user = \'\')'
