@@ -3,7 +3,8 @@
 /**
  * Common Option Constants For DBI Functions
  *
- * @version $Id: database_interface.lib.php 11585 2008-09-15 12:03:45Z lem9 $
+ * @version $Id$
+ * @package phpMyAdmin
  */
 if (! defined('PHPMYADMIN')) {
     exit;
@@ -16,11 +17,11 @@ if (! defined('PHPMYADMIN')) {
 define('PMA_DBI_QUERY_STORE',       1);  // Force STORE_RESULT method, ignored by classic MySQL.
 define('PMA_DBI_QUERY_UNBUFFERED',  2);  // Do not read whole query
 // PMA_DBI_get_variable()
-define('PMA_DBI_GETVAR_SESSION', 1);
-define('PMA_DBI_GETVAR_GLOBAL', 2);
+define('PMA_DBI_GETVAR_SESSION',    1);
+define('PMA_DBI_GETVAR_GLOBAL',     2);
 
 /**
- * Checks one of the mysql extensions 
+ * Checks one of the mysql extensions
  *
  * @param   string  $extension  mysql extension to check
  */
@@ -32,7 +33,6 @@ function PMA_DBI_checkMysqlExtension($extension = 'mysql') {
     return true;
 }
 
-
 /**
  * check for requested extension
  */
@@ -79,8 +79,8 @@ require_once './libraries/dbi/' . $GLOBALS['cfg']['Server']['extension'] . '.dbi
 /**
  * Common Functions
  */
-function PMA_DBI_query($query, $link = null, $options = 0) {
-    $res = PMA_DBI_try_query($query, $link, $options)
+function PMA_DBI_query($query, $link = null, $options = 0, $cache_affected_rows = true) {
+    $res = PMA_DBI_try_query($query, $link, $options, $cache_affected_rows)
         or PMA_mysqlDie(PMA_DBI_getError($link), $query);
     return $res;
 }
@@ -187,8 +187,8 @@ function PMA_DBI_get_tables($database, $link = null)
 /**
  * usort comparison callback
  *
- * @param   string  $a first argument to sort 
- * @param   string  $b second argument to sort 
+ * @param   string  $a first argument to sort
+ * @param   string  $b second argument to sort
  *
  * @return  integer  a value representing whether $a should be before $b in the
  *                   sorted array or not
@@ -205,6 +205,10 @@ function PMA_usort_comparison_callback($a, $b)
     } else {
         $sorter = 'strcasecmp';
     }
+    /* No sorting when key is not present */
+    if (!isset($a[$GLOBALS['callback_sort_by']]) || ! isset($b[$GLOBALS['callback_sort_by']])) {
+        return 0;
+    }
     // produces f.e.:
     // return -1 * strnatcasecmp($a["SCHEMA_TABLES"], $b["SCHEMA_TABLES"])
     return ($GLOBALS['callback_sort_order'] == 'ASC' ? 1 : -1) * $sorter($a[$GLOBALS['callback_sort_by']], $b[$GLOBALS['callback_sort_by']]);
@@ -240,13 +244,15 @@ function PMA_usort_comparison_callback($a, $b)
  * @param   resource        $link           mysql link
  * @param   integer         $limit_offset   zero-based offset for the count
  * @param   boolean|integer $limit_count    number of tables to return
+ * @param   string          $sort_by        table attribute to sort by
+ * @param   string          $sort_order     direction to sort (ASC or DESC)
  * @return  array           list of tables in given db(s)
  */
-function PMA_DBI_get_tables_full($database, $table = false,
-    $tbl_is_group = false, $link = null, $limit_offset = 0, $limit_count = false)
+function PMA_DBI_get_tables_full($database, $table = false, $tbl_is_group = false, $link = null,
+     $limit_offset = 0, $limit_count = false, $sort_by = 'Name', $sort_order = 'ASC')
 {
     require_once './libraries/Table.class.php';
-    
+
     if (true === $limit_count) {
         $limit_count = $GLOBALS['cfg']['MaxTableList'];
     }
@@ -261,19 +267,19 @@ function PMA_DBI_get_tables_full($database, $table = false,
 
     if (! $GLOBALS['cfg']['Server']['DisableIS']) {
       // get table information from information_schema
-      if ($table) {
-          if (true === $tbl_is_group) {
-              $sql_where_table = 'AND `TABLE_NAME` LIKE \''
+        if ($table) {
+            if (true === $tbl_is_group) {
+                $sql_where_table = 'AND `TABLE_NAME` LIKE \''
                   . PMA_escape_mysql_wildcards(addslashes($table)) . '%\'';
-          } elseif ('comment' === $tbl_is_group) {
-              $sql_where_table = 'AND `TABLE_COMMENT` LIKE \''
+            } elseif ('comment' === $tbl_is_group) {
+                $sql_where_table = 'AND `TABLE_COMMENT` LIKE \''
                   . PMA_escape_mysql_wildcards(addslashes($table)) . '%\'';
-          } else {
-              $sql_where_table = 'AND `TABLE_NAME` = \'' . addslashes($table) . '\'';
-          }
-      } else {
-          $sql_where_table = '';
-      }
+            } else {
+                $sql_where_table = 'AND `TABLE_NAME` = \'' . addslashes($table) . '\'';
+            }
+        } else {
+            $sql_where_table = '';
+        }
 
       // for PMA bc:
       // `SCHEMA_FIELD_NAME` AS `SHOW_TABLE_STATUS_FIELD_NAME`
@@ -282,12 +288,13 @@ function PMA_DBI_get_tables_full($database, $table = false,
       // added BINARY in the WHERE clause to force a case sensitive
       // comparison (if we are looking for the db Aa we don't want
       // to find the db aa)
-      $this_databases = array_map('PMA_sqlAddslashes', $databases);
+        $this_databases = array_map('PMA_sqlAddslashes', $databases);
 
-      $sql = '
+        $sql = '
            SELECT *,
                   `TABLE_SCHEMA`       AS `Db`,
                   `TABLE_NAME`         AS `Name`,
+                  `TABLE_TYPE`         ÀS `TABLE_TYPE`,
                   `ENGINE`             AS `Engine`,
                   `ENGINE`             AS `Type`,
                   `VERSION`            AS `Version`,
@@ -310,20 +317,35 @@ function PMA_DBI_get_tables_full($database, $table = false,
             WHERE ' . (PMA_IS_WINDOWS ? '' : 'BINARY') . ' `TABLE_SCHEMA` IN (\'' . implode("', '", $this_databases) . '\')
               ' . $sql_where_table;
 
-      if ($limit_count) {
-          $sql .= ' LIMIT ' . $limit_count . ' OFFSET ' . $limit_offset;
-      }
-      $tables = PMA_DBI_fetch_result($sql, array('TABLE_SCHEMA', 'TABLE_NAME'),
+        // Sort the tables
+        $sql .= " ORDER BY $sort_by $sort_order";
+
+        if ($limit_count) {
+            $sql .= ' LIMIT ' . $limit_count . ' OFFSET ' . $limit_offset;
+        }
+
+        $tables = PMA_DBI_fetch_result($sql, array('TABLE_SCHEMA', 'TABLE_NAME'),
           null, $link);
-      unset($sql_where_table, $sql);
-    }
+        unset($sql_where_table, $sql);
+        if ($sort_by == 'Name' && $GLOBALS['cfg']['NaturalOrder']) {
+            // here, the array's first key is by schema name
+            foreach($tables as $one_database_name => $one_database_tables) {
+                uksort($one_database_tables, 'strnatcasecmp');
+
+                if ($sort_order == 'DESC') {
+                    $one_database_tables = array_reverse($one_database_tables);
+                }
+                $tables[$one_database_name] = $one_database_tables;
+            }
+        }
+    } // end (get information from table schema)
 
     // If permissions are wrong on even one database directory,
     // information_schema does not return any table info for any database
     // this is why we fall back to SHOW TABLE STATUS even for MySQL >= 50002
     if (empty($tables)) {
         foreach ($databases as $each_database) {
-            if (true === $tbl_is_group) {
+            if ($table || (true === $tbl_is_group)) {
                 $sql = 'SHOW TABLE STATUS FROM '
                     . PMA_backquote($each_database)
                     .' LIKE \'' . PMA_escape_mysql_wildcards(addslashes($table)) . '%\'';
@@ -334,6 +356,39 @@ function PMA_DBI_get_tables_full($database, $table = false,
 
             $each_tables = PMA_DBI_fetch_result($sql, 'Name', null, $link);
 
+            // Sort naturally if the config allows it and we're sorting
+            // the Name column.
+            if ($sort_by == 'Name' && $GLOBALS['cfg']['NaturalOrder']) {
+                uksort($each_tables, 'strnatcasecmp');
+
+                if ($sort_order == 'DESC') {
+                    $each_tables = array_reverse($each_tables);
+                }
+            } else {
+                // Prepare to sort by creating array of the selected sort
+                // value to pass to array_multisort
+
+                // Size = Data_length + Index_length
+                if ($sort_by == 'Data_length') {
+                    foreach ($each_tables as $table_name => $table_data) {
+                        ${$sort_by}[$table_name] = strtolower($table_data['Data_length'] + $table_data['Index_length']);
+                    }
+                } else {
+                    foreach ($each_tables as $table_name => $table_data) {
+                        ${$sort_by}[$table_name] = strtolower($table_data[$sort_by]);
+                    }
+                }
+
+                if ($sort_order == 'DESC') {
+                    array_multisort($$sort_by, SORT_DESC, $each_tables);
+                } else {
+                    array_multisort($$sort_by, SORT_ASC, $each_tables);
+                }
+
+                // cleanup the temporary sort array
+                unset($$sort_by);
+            }
+
             if ($limit_count) {
                 $each_tables = array_slice($each_tables, $limit_offset, $limit_count);
             }
@@ -381,7 +436,8 @@ function PMA_DBI_get_tables_full($database, $table = false,
                 $each_tables[$table_name]['CREATE_OPTIONS']    =& $each_tables[$table_name]['Create_options'];
                 $each_tables[$table_name]['TABLE_COMMENT']     =& $each_tables[$table_name]['Comment'];
 
-                if (strtoupper($each_tables[$table_name]['Comment']) === 'VIEW') {
+                if (strtoupper($each_tables[$table_name]['Comment']) === 'VIEW'
+                        && $each_tables[$table_name]['Engine'] == NULL) {
                     $each_tables[$table_name]['TABLE_TYPE'] = 'VIEW';
                 } else {
                     /**
@@ -395,15 +451,22 @@ function PMA_DBI_get_tables_full($database, $table = false,
         }
     }
 
-    if ($GLOBALS['cfg']['NaturalOrder']) {
-        foreach ($tables as $key => $val) {
-            uksort($tables[$key], 'strnatcasecmp');
-        }
-    }
-    
     // cache table data
     // so PMA_Table does not require to issue SHOW TABLE STATUS again
-    PMA_Table::$cache = $tables;
+    // Note: I don't see why we would need array_merge_recursive() here,
+    // as it creates double entries for the same table (for example a double
+    // entry for Comment when changing the storage engine in Operations)
+    // Note 2: Instead of array_merge(), simply use the + operator because
+    //  array_merge() renumbers numeric keys starting with 0, therefore
+    //  we would lose a db name thats consists only of numbers
+    foreach($tables as $one_database => $its_tables) {
+        if (isset(PMA_Table::$cache[$one_database])) {
+            PMA_Table::$cache[$one_database] = PMA_Table::$cache[$one_database] + $tables[$one_database];
+        } else {
+            PMA_Table::$cache[$one_database] = $tables[$one_database];
+        }
+    }
+    unset($one_database, $its_tables);
 
     if (! is_array($database)) {
         if (isset($tables[$database])) {
@@ -519,7 +582,7 @@ function PMA_DBI_get_databases_full($database = null, $force_stats = false,
 
         // display only databases also in official database list
         // f.e. to apply hide_db and only_db
-        $drops = array_diff(array_keys($databases), $GLOBALS['pma']->databases);
+        $drops = array_diff(array_keys($databases), (array) $GLOBALS['pma']->databases);
         if (count($drops)) {
             foreach ($drops as $drop) {
                 unset($databases[$drop]);
@@ -534,7 +597,7 @@ function PMA_DBI_get_databases_full($database = null, $force_stats = false,
             $databases[$database_name]['SCHEMA_NAME']      = $database_name;
 
             if ($force_stats) {
-                require_once 'mysql_charsets.lib.php';
+                require_once './libraries/mysql_charsets.lib.php';
 
                 $databases[$database_name]['DEFAULT_COLLATION_NAME']
                     = PMA_getDbCollation($database_name);
@@ -559,8 +622,13 @@ function PMA_DBI_get_databases_full($database = null, $force_stats = false,
                         += $row['Max_data_length'];
                     $databases[$database_name]['SCHEMA_INDEX_LENGTH']
                         += $row['Index_length'];
-                    $databases[$database_name]['SCHEMA_DATA_FREE']
-                        += $row['Data_free'];
+
+                    // for InnoDB, this does not contain the number of 
+                    // overhead bytes but the total free space
+                    if ('InnoDB' != $row['Engine']) {
+                        $databases[$database_name]['SCHEMA_DATA_FREE']
+                            += $row['Data_free'];
+                    }
                     $databases[$database_name]['SCHEMA_LENGTH']
                         += $row['Data_length'] + $row['Index_length'];
                 }
@@ -764,6 +832,79 @@ function PMA_DBI_get_columns($database, $table, $full = false, $link = null)
 }
 
 /**
+ * array PMA_DBI_get_column_values (string $database, string $table, string $column , mysql db link $link = null)
+ *
+ * @param   string  $database   name of database
+ * @param   string  $table      name of table to retrieve columns from
+ * @param   string  $column     name of the column to retrieve data from 
+ * @param   mixed   $link       mysql link resource
+ * @return  array   $field_values
+ */
+
+function PMA_DBI_get_column_values($database, $table, $column, $link = null)
+{ 
+    $query = 'SELECT ';
+    for($i=0; $i< sizeof($column); $i++)
+    {
+        $query.= PMA_backquote($column[$i]);
+        if($i < (sizeof($column)-1))
+        {
+            $query.= ', ';   
+        }
+    }
+    $query.= ' FROM ' . PMA_backquote($database) . '.' . PMA_backquote($table);
+    $field_values = PMA_DBI_fetch_result($query, null, null, $link);
+    
+    if (! is_array($field_values) || count($field_values) < 1) {
+        return false;
+    }
+    return $field_values;
+}
+/**
+ * array PMA_DBI_get_table_data (string $database, string $table, mysql db link $link = null)
+ *
+ * @param   string  $database   name of database
+ * @param   string  $table      name of table to retrieve columns from
+ * @param   mixed   $link       mysql link resource
+ * @return  array   $result
+ */
+ 
+ function PMA_DBI_get_table_data($database, $table, $link = null)
+ { 
+                           
+    $result = PMA_DBI_fetch_result(
+        'SELECT * FROM ' . PMA_backquote($database) . '.' . PMA_backquote($table),
+        null,null, $link);
+                           
+    if (! is_array($result) || count($result) < 1) {
+        return false;
+    }                                                                                         
+    return $result;
+ }
+ 
+/**
+* array  PMA_DBI_get_table_indexes($database, $table, $link = null)
+* 
+* @param    string  $database   name of database
+* @param    string  $table      name of the table whose indexes are to be retreived
+* @param    mixed   $link       mysql link resource
+* @return   array   $indexes
+*/
+
+function PMA_DBI_get_table_indexes($database, $table, $link = null)
+{
+ 
+    $indexes = PMA_DBI_fetch_result(
+              'SHOW INDEXES FROM ' .PMA_backquote($database) . '.' . PMA_backquote($table),
+               null, null, $link);  
+    
+    if (! is_array($indexes) || count($indexes) < 1) {
+        return false;
+    }
+    return $indexes;
+}
+ 
+ /**
  * returns value of given mysql server variable
  *
  * @param   string  $var    mysql server variable name
@@ -771,6 +912,8 @@ function PMA_DBI_get_columns($database, $table, $full = false, $link = null)
  * @param   mixed   $link   mysql link resource|object
  * @return  mixed   value for mysql server variable
  */
+ 
+ 
 function PMA_DBI_get_variable($var, $type = PMA_DBI_GETVAR_SESSION, $link = null)
 {
     if ($link === null) {
@@ -796,6 +939,9 @@ function PMA_DBI_get_variable($var, $type = PMA_DBI_GETVAR_SESSION, $link = null
 }
 
 /**
+ *  Function called just after a connection to the MySQL database server has been established
+ *  It sets the connection collation, and determins the version of MySQL which is running.
+ *
  * @uses    ./libraries/charset_conversion.lib.php
  * @uses    PMA_DBI_QUERY_STORE
  * @uses    PMA_MYSQL_INT_VERSION to set it
@@ -850,9 +996,9 @@ function PMA_DBI_postConnect($link, $is_controluser = false)
     }
 
     if (! empty($GLOBALS['collation_connection'])) {
+    	PMA_DBI_query("SET CHARACTER SET 'utf8';", $link, PMA_DBI_QUERY_STORE);
         $mysql_charset = explode('_', $GLOBALS['collation_connection']);
-        PMA_DBI_query("SET NAMES '" . $mysql_charset[0] . "' COLLATE '" . $GLOBALS['collation_connection'] . "';",
-            $link, PMA_DBI_QUERY_STORE);
+        PMA_DBI_query("SET collation_connection = '" . PMA_sqlAddslashes($GLOBALS['collation_connection']) . "';", $link, PMA_DBI_QUERY_STORE);
     } else {
         PMA_DBI_query("SET NAMES 'utf8' COLLATE 'utf8_general_ci';", $link, PMA_DBI_QUERY_STORE);
     }
@@ -1239,7 +1385,7 @@ function PMA_DBI_get_procedures_or_functions($db, $which, $link = null)
 function PMA_DBI_get_definition($db, $which, $name, $link = null)
 {
     $returned_field = array(
-        'PROCEDURE' => 'Create Procedure', 
+        'PROCEDURE' => 'Create Procedure',
         'FUNCTION'  => 'Create Function',
         'EVENT'     => 'Create Event'
     );
@@ -1253,33 +1399,66 @@ function PMA_DBI_get_definition($db, $which, $name, $link = null)
  * @uses    PMA_DBI_fetch_result()
  * @param   string              $db     db name
  * @param   string              $table  table name
+ * @param   string              $delimiter  the delimiter to use (may be empty) 
  *
  * @return  array               information about triggers (may be empty)
  */
-function PMA_DBI_get_triggers($db, $table)
+function PMA_DBI_get_triggers($db, $table, $delimiter = '//')
 {
     $result = array();
 
+    if (! $GLOBALS['cfg']['Server']['DisableIS']) {
     // Note: in http://dev.mysql.com/doc/refman/5.0/en/faqs-triggers.html
     // their example uses WHERE TRIGGER_SCHEMA='dbname' so let's use this
     // instead of WHERE EVENT_OBJECT_SCHEMA='dbname'
-    $triggers = PMA_DBI_fetch_result("SELECT TRIGGER_SCHEMA, TRIGGER_NAME, EVENT_MANIPULATION, ACTION_TIMING, ACTION_STATEMENT, EVENT_OBJECT_SCHEMA, EVENT_OBJECT_TABLE FROM information_schema.TRIGGERS WHERE TRIGGER_SCHEMA= '" . PMA_sqlAddslashes($db,true) . "' and EVENT_OBJECT_TABLE = '" . PMA_sqlAddslashes($table, true) . "';");
+        $triggers = PMA_DBI_fetch_result("SELECT TRIGGER_SCHEMA, TRIGGER_NAME, EVENT_MANIPULATION, ACTION_TIMING, ACTION_STATEMENT, EVENT_OBJECT_SCHEMA, EVENT_OBJECT_TABLE FROM information_schema.TRIGGERS WHERE TRIGGER_SCHEMA= '" . PMA_sqlAddslashes($db,true) . "' and EVENT_OBJECT_TABLE = '" . PMA_sqlAddslashes($table, true) . "';");
+    } else {
+        $triggers = PMA_DBI_fetch_result("SHOW TRIGGERS FROM " . PMA_backquote(PMA_sqlAddslashes($db,true)) . " LIKE '" . PMA_sqlAddslashes($table, true) . "';");
+    }
 
     if ($triggers) {
-        $delimiter = '//';
         foreach ($triggers as $trigger) {
+            if ($GLOBALS['cfg']['Server']['DisableIS']) {
+                $trigger['TRIGGER_NAME'] = $trigger['Trigger'];
+                $trigger['ACTION_TIMING'] = $trigger['Timing'];
+                $trigger['EVENT_MANIPULATION'] = $trigger['Event'];
+                $trigger['EVENT_OBJECT_TABLE'] = $trigger['Table'];
+                $trigger['ACTION_STATEMENT'] = $trigger['Statement'];
+            }
             $one_result = array();
             $one_result['name'] = $trigger['TRIGGER_NAME'];
             $one_result['action_timing'] = $trigger['ACTION_TIMING'];
             $one_result['event_manipulation'] = $trigger['EVENT_MANIPULATION'];
 
-            $one_result['full_trigger_name'] = PMA_backquote($trigger['TRIGGER_SCHEMA']) . '.' . PMA_backquote($trigger['TRIGGER_NAME']);
+            // do not prepend the schema name; this way, importing the
+            // definition into another schema will work
+            $one_result['full_trigger_name'] = PMA_backquote($trigger['TRIGGER_NAME']);
             $one_result['drop'] = 'DROP TRIGGER IF EXISTS ' . $one_result['full_trigger_name'];
-            $one_result['create'] = 'CREATE TRIGGER ' . $one_result['full_trigger_name'] . ' ' . $trigger['ACTION_TIMING']. ' ' . $trigger['EVENT_MANIPULATION'] . ' ON ' . PMA_backquote($trigger['EVENT_OBJECT_SCHEMA']) . '.' . PMA_backquote($trigger['EVENT_OBJECT_TABLE']) . "\n" . ' FOR EACH ROW ' . $trigger['ACTION_STATEMENT'] . "\n" . $delimiter . "\n";
+            $one_result['create'] = 'CREATE TRIGGER ' . $one_result['full_trigger_name'] . ' ' . $trigger['ACTION_TIMING']. ' ' . $trigger['EVENT_MANIPULATION'] . ' ON ' . PMA_backquote($trigger['EVENT_OBJECT_TABLE']) . "\n" . ' FOR EACH ROW ' . $trigger['ACTION_STATEMENT'] . "\n" . $delimiter . "\n";
 
             $result[] = $one_result;
         }
     }
     return($result);
 }
+
+/**
+ * Returns TRUE if $db.$view_name is a view, FALSE if not
+ *
+ * @uses   PMA_DBI_fetch_result()
+ * @param  string $db         database name
+ * @param  string $view_name  view/table name
+ *
+ * @return bool               TRUE if $db.$view_name is a view, FALSE if not
+ */
+function PMA_isView($db, $view_name)
+{
+    $result = PMA_DBI_fetch_result("SELECT TABLE_NAME FROM information_schema.VIEWS WHERE TABLE_SCHEMA = '".$db."' and TABLE_NAME = '".$view_name."';");
+    
+    if ($result) {
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
 ?>
