@@ -4,7 +4,7 @@
  * Interface to the classic MySQL extension
  *
  * @package phpMyAdmin-DBI-MySQL
- * @version $Id: mysql.dbi.lib.php 12283 2009-03-03 16:20:41Z nijel $
+ * @version $Id$
  */
 if (! defined('PHPMYADMIN')) {
     exit;
@@ -21,18 +21,18 @@ if (! defined('PMA_MYSQL_CLIENT_API')) {
     unset($client_api);
 }
 
-function PMA_DBI_real_connect($server, $user, $password, $client_flags)
+function PMA_DBI_real_connect($server, $user, $password, $client_flags, $persistant=false)
 {
     global $cfg;
 
     if (empty($client_flags)) {
-        if ($cfg['PersistentConnections']) {
+        if ($cfg['PersistentConnections'] || $persistant) {
             $link = @mysql_pconnect($server, $user, $password);
         } else {
             $link = @mysql_connect($server, $user, $password);
         }
     } else {
-        if ($cfg['PersistentConnections']) {
+        if ($cfg['PersistentConnections'] || $persistant) {
             $link = @mysql_pconnect($server, $user, $password, $client_flags);
         } else {
             $link = @mysql_connect($server, $user, $password, false, $client_flags);
@@ -41,23 +41,41 @@ function PMA_DBI_real_connect($server, $user, $password, $client_flags)
 
     return $link;
 }
-
-function PMA_DBI_connect($user, $password, $is_controluser = false)
+/**
+ * @param   string  $user           mysql user name
+ * @param   string  $password       mysql user password
+ * @param   boolean $is_controluser
+ * @param   array   $server host/port/socket/persistant
+ * @param   boolean $auxiliary_connection (when true, don't go back to login if connection fails)
+ * @return  mixed   false on error or a mysqli object on success
+ */
+function PMA_DBI_connect($user, $password, $is_controluser = false, $server = null, $auxiliary_connection = false)
 {
     global $cfg, $php_errormsg;
-
-    $server_port   = (empty($cfg['Server']['port']))
+  
+    if ($server) {
+        $server_port = (empty($server['port']))
+            ? ''
+            : ':' . (int)$server['port'];
+        $server_socket = (empty($server['socket']))
+            ? ''
+            : ':' . $server['socket'];
+        $server_persistant = (empty($server['persistant']))
+            ? false
+            : true;
+    } else {
+	  $server_port   = (empty($cfg['Server']['port']))
                    ? ''
-                   : ':' . $cfg['Server']['port'];
+                   : ':' . (int)$cfg['Server']['port'];
+	  $server_socket = (empty($cfg['Server']['socket']))
+                   ? ''
+                   : ':' . $cfg['Server']['socket'];
+    }
 
     if (strtolower($cfg['Server']['connect_type']) == 'tcp') {
         $cfg['Server']['socket'] = '';
     }
 
-    $server_socket = (empty($cfg['Server']['socket']))
-                   ? ''
-                   : ':' . $cfg['Server']['socket'];
-
     $client_flags = 0;
 
     // always use CLIENT_LOCAL_FILES as defined in mysql_com.h
@@ -74,25 +92,39 @@ function PMA_DBI_connect($user, $password, $is_controluser = false)
     if (defined('MYSQL_CLIENT_SSL') && $cfg['Server']['ssl']) {
         $client_flags |= MYSQL_CLIENT_SSL;
     }
+    
+    if (!$server) {
+        $link = PMA_DBI_real_connect($cfg['Server']['host'] . $server_port . $server_socket, $user, $password, empty($client_flags) ? NULL : $client_flags);
 
-    $link = PMA_DBI_real_connect($cfg['Server']['host'] . $server_port . $server_socket, $user, $password, empty($client_flags) ? NULL : $client_flags);
-
-    // Retry with empty password if we're allowed to
-    if (empty($link) && $cfg['Server']['nopassword'] && !$is_controluser) {
-        $link = PMA_DBI_real_connect($cfg['Server']['host'] . $server_port . $server_socket, $user, '', empty($client_flags) ? NULL : $client_flags);
+      // Retry with empty password if we're allowed to
+        if (empty($link) && $cfg['Server']['nopassword'] && !$is_controluser) {
+	        $link = PMA_DBI_real_connect($cfg['Server']['host'] . $server_port . $server_socket, $user, '', empty($client_flags) ? NULL : $client_flags);
+        }
+    } else {
+        if (!isset($server['host'])) {
+	        $link = PMA_DBI_real_connect($server_socket, $user, $password, NULL, $server_persistant); 
+        } else {
+            $link = PMA_DBI_real_connect($server['host'] . $server_port . $server_socket, $user, $password, NULL, $server_persistant);
+        }
     }
-
     if (empty($link)) {
         if ($is_controluser) {
             trigger_error($GLOBALS['strControluserFailed'], E_USER_WARNING);
             return false;
         }
-        PMA_log_user($user, 'mysql-denied');
-        PMA_auth_fails();
+        // we could be calling PMA_DBI_connect() to connect to another
+        // server, for example in the Synchronize feature, so do not
+        // go back to main login if it fails
+        if (! $auxiliary_connection) {
+            PMA_log_user($user, 'mysql-denied');
+            PMA_auth_fails();
+        } else {
+            return false;
+        }
     } // end if
-
-    PMA_DBI_postConnect($link, $is_controluser);
-
+    if (! $server) {
+        PMA_DBI_postConnect($link, $is_controluser);
+    }
     return $link;
 }
 
@@ -123,7 +155,7 @@ function PMA_DBI_select_db($dbname, $link = null)
  * @param integer $options
  * @return mixed
  */
-function PMA_DBI_try_query($query, $link = null, $options = 0)
+function PMA_DBI_try_query($query, $link = null, $options = 0, $cache_affected_rows = true)
 {
     if (empty($link)) {
         if (isset($GLOBALS['userlink'])) {
@@ -144,6 +176,10 @@ function PMA_DBI_try_query($query, $link = null, $options = 0)
         $r = mysql_query($query, $link);
     }
 
+    if ($cache_affected_rows) { 
+       $GLOBALS['cached_affected_rows'] = PMA_DBI_affected_rows($link, $get_from_cache = false); 
+    }
+
     if ($GLOBALS['cfg']['DBG']['sql']) {
         $time = microtime(true) - $time;
 
@@ -173,6 +209,9 @@ function PMA_DBI_try_query($query, $link = null, $options = 0)
         }
         $_SESSION['debug']['queries'][$hash]['trace'][] = $trace;
     }
+    if ($r != FALSE && PMA_Tracker::isActive() == TRUE ) {
+        PMA_Tracker::handleQuery($query); 
+    }
 
     return $r;
 }
@@ -286,6 +325,12 @@ function PMA_DBI_get_client_info()
 function PMA_DBI_getError($link = null)
 {
     $GLOBALS['errno'] = 0;
+
+    /* Treat false same as null because of controllink */
+    if ($link === false) {
+        $link = null;
+    }
+
     if (null === $link && isset($GLOBALS['userlink'])) {
         $link =& $GLOBALS['userlink'];
 
@@ -313,6 +358,8 @@ function PMA_DBI_getError($link = null)
         $error_message = PMA_DBI_convert_message($error_message);
     }
 
+    $error_message = htmlspecialchars($error_message);
+
     // Some errors messages cannot be obtained by mysql_error()
     if ($error_number == 2002) {
         $error = '#' . ((string) $error_number) . ' - ' . $GLOBALS['strServerNotResponding'] . ' ' . $GLOBALS['strSocketProblem'];
@@ -349,17 +396,24 @@ function PMA_DBI_insert_id($link = null)
             return false;
         }
     }
-    //$insert_id = mysql_insert_id($link);
-    // if the primary key is BIGINT we get an incorrect result
+    // If the primary key is BIGINT we get an incorrect result
     // (sometimes negative, sometimes positive)
     // and in the present function we don't know if the PK is BIGINT
     // so better play safe and use LAST_INSERT_ID()
     //
-    // by the way, no problem with mysqli_insert_id()
     return PMA_DBI_fetch_value('SELECT LAST_INSERT_ID();', 0, 0, $link);
 }
 
-function PMA_DBI_affected_rows($link = null)
+/**
+ * returns the number of rows affected by last query
+ *
+ * @uses    $GLOBALS['userlink']
+ * @uses    mysql_affected_rows()
+ * @param   object mysql   $link   the mysql object
+ * @param   boolean        $get_from_cache 
+ * @return  string integer
+ */
+function PMA_DBI_affected_rows($link = null, $get_from_cache = true)
 {
     if (empty($link)) {
         if (isset($GLOBALS['userlink'])) {
@@ -368,7 +422,12 @@ function PMA_DBI_affected_rows($link = null)
             return false;
         }
     }
-    return mysql_affected_rows($link);
+
+    if ($get_from_cache) {
+        return $GLOBALS['cached_affected_rows'];
+    } else {
+        return mysql_affected_rows($link);
+    }
 }
 
 /**
